from pvw.config import config
import os
from pvw.shell_executor import ShellExecutor
from termcolor import colored


def _get_directory_size(path="."):
    total_size = 0
    for dirpath, _, filenames in os.walk(path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            # skip if it is symbolic link
            if not os.path.islink(fp):
                total_size += os.path.getsize(fp)

    return total_size / (1024 * 1024)


class Environment:
    def __init__(self, name, path):
        self.name = name
        self.path = path
        self.size = ""  # lazy load

    def set_size(self, size):
        self.size = size

    def to_list(self):
        if self.size:
            return [self.name, self.path, self.size]
        return [self.name, self.path]


class EnvironmentManager:
    """
    Manage and log the principle information about environment, feed then to ShellExecutor to communicate with system.
    """

    def __init__(self):
        self.shell = ShellExecutor()
        self._envs = self.load()
        self.show_size = False

    def load(self):
        config.build_venv_path()
        envs = {}
        venv_info = config.get("venvs")
        for info in venv_info:
            name = info["name"]
            path = info["path"]
            env = Environment(name, path)
            envs[name] = env
        return envs

    def read_size(self):
        self.show_size = True
        for name in self._envs:
            path = os.path.join(config.venv_path, name)
            size = "{:.2f}MB".format(_get_directory_size(path=path))
            self._envs[name].size = size

    # Operations to show or check envs

    def show(self, envs=[]):
        if self.show_size:
            header = ["Name", "Path", "Size"]
        else:
            header = ["Name", "Path"]
        if envs:
            env_list = [self._envs[env].to_list() for env in envs if self.name_exists(env)]
        else:
            env_list = [self._envs[name].to_list() for name in self._envs]
        col_widths = [
            max(len(str(x)) + 2 for x in col) for col in zip(header, *env_list)
        ]
        print()
        for i, column in enumerate(header):
            print(f"{column:<{col_widths[i]}}", end=" ")
        print()
        print(colored("-" * sum(col_widths), "green"))
        if env_list:
            for row in env_list:
                for i, cell in enumerate(row):
                    print(f"{cell:<{col_widths[i]}}", end=" ")
                print()
        else:
            print("None")
        print()

    def check_python_installation(self):
        return self.shell.check_python_installation()

    def name_exists(self, name):
        return name in self._envs

    def path_exists(self, path):
        return path in [self._envs[name].path for name in self._envs]

    def check_not_exists(self, name):
        path, name = self.norm_path(name)
        if self.name_exists(name):
            raise NameError(f"Environment named `{name}` has already exists.")
        if self.path_exists(path):
            raise NameError(f"Path {path} has already exists.")

    def check_exists(self, name):
        if not self.name_exists(name):
            raise NameError(f"Environment `{name}` does not exists.")

    @staticmethod
    def norm_path(path):
        if path.startswith("."):
            path = os.path.join(os.getcwd(), os.path.normpath(path))
        path = os.path.normpath(path)
        name = os.path.basename(path)

        if name == path:  # just type a name
            return os.path.join(config.venv_path, name), name
        if os.path.isabs(path):
            return path, name
        return os.path.join(os.getcwd(), path), name

    @staticmethod
    def split_path_name(path):
        return os.path.dirname(path), os.path.basename(path)

    # Operations to modify envs

    def create(self, name):
        path, name = self.norm_path(name)
        self.check_not_exists(name)
        print(f"creating {name} in {path}...")
        self.shell.create_env(path)
        config.add_venv(name, path)
        print(f"{name} created successfully.")

    def activate(self, name):
        self.check_exists(name)
        path = self._envs[name].path
        self.shell.activate_env(path)

    def remove(self, names):
        valid_names = []
        for name in names:
            try:
                self.check_exists(name)
                valid_names.append(name)
            except NameError as e:
                print(colored(e, "yellow"))
        if not valid_names:
            return
        self.show(envs=valid_names)
        confirm = input(
            colored(f"Sure to remove `{','.join(valid_names)}`? [y/N]", "red")
        )
        if confirm.lower() == "y":
            for name in valid_names:
                env = self._envs[name]
                path = env.path
                self.shell.remove_env(path)

    def copy(self, source, target):
        self.check_exists(source)
        self.check_not_exists(target)
        path, name = self.split_path_name(target)
        print("Start copying...")
        source_path = self._envs[source].path
        if path:
            target_path = target
        else:
            target_path = os.path.join(config.venv_path, name)
        self.shell.copy_env(source_path, target_path)
        config.add_venv(name, target_path)
        print(
            colored(
                f"venv `{target}` has been successfully copied from `{source}`", "green"
            )
        )

    def move(self, source, target):
        self.check_exists(source)
        self.check_not_exists(target)
        self.show(envs=[source])
        confirm = input(f"Sure to move venv `{source}` to `{target}`? [y/N]")
        if confirm.lower() == "y":
            path, name = self.split_path_name(target)
            print("Start moving...")
            source_path = self._envs[source].path
            if path:
                target_path = target
            else:
                target_path = os.path.join(config.venv_path, name)
            self.shell.copy_env(source_path, target_path)
            self.shell.remove_env(source_path)
            config.add_venv(name, target_path)
            config.remove_venv(names=[source])
            print(
                colored(
                    f"venv `{target}` has been successfully moved from `{source}`",
                    "green",
                )
            )



from pvw.env import EnvironmentManager
from pvw.config import config
from pvw.template import Template
import os
import click

_env_manager = EnvironmentManager()


@click.group
@click.version_option()
def pvw():
    pass


@pvw.command
@click.option(
    "-s",
    "--show-size",
    default=False,
    is_flag=True,
    help="whether show sizes of each venv, this operation could take a while.",
)
def ls(show_size):
    """
    list all venvs.
    """
    if show_size:
        _env_manager.read_size()
    _env_manager.show()


@pvw.command()
@click.argument("name")
def create(name):
    """
    create a new venv
    """
    _env_manager.create(name=name)


@pvw.command()
@click.argument("names", nargs=-1)
def rm(names):
    """
    remove an existing venv
    """
    _env_manager.remove(names=names)


@pvw.command()
@click.argument("name")
@click.option(
    "--shorten",
    hidden=True,
    is_flag=True,
    help="whether using a shorten stype for activating envs. (hidden option)",
)
def activate(name, shorten):
    """
    activate a venv. (additional `source` command is needed in Linux/MacOS)
    """
    try:
        _env_manager.activate(name=name)
    except NameError as e:
        if shorten:
            ctx = pvw.make_context("pvw", [name])
            with ctx:
                pvw.invoke(ctx)
        else:
            click.echo(e)


@pvw.command()
@click.argument("name")
def init(name):
    """
    start a new project with default template
    """
    try:
        path = os.path.join(os.getcwd(), name)
        template = Template(name=name, path=path)
        template.build()
        is_create_venv = click.confirm(
            "Do you want to create a new venv for this project?", default=True
        )
        if is_create_venv:
            venv_name = input(f"Enter the name of venv ({name})")
            if not venv_name:
                venv_name = name
            _env_manager.create(os.path.join(path, name))
    except Exception as e:
        print(e)


@pvw.command()
@click.argument("src")
@click.argument("dest")
def mv(src, dest):
    """
    move (or rename) venv `src` to `dest`
    """
    _env_manager.move(source=src, target=dest)


@pvw.command()
@click.argument("src")
@click.argument("dest")
def cp(src, dest):
    """
    copy venv `src` to `dest`
    """
    _env_manager.copy(source=src, target=dest)


@pvw.group(name="config")
def config_cli():
    """get or set variables in config"""
    pass


@config_cli.command
@click.argument("name")
@click.argument("value")
def set(name, value):
    """
    set config options, including `venv_path`.
    """
    if name == "venv_path":
        config.set(name, value)


@config_cli.command
@click.argument("name")
def get(name):
    """
    get config options, including `venv_path`
    """
    if name == "venv_path":
        path = config.get(name)
        click.echo(path)


def parse():
    pvw(prog_name="pvw")


if __name__ == "__main__":
    parse()



import os
from termcolor import colored
from datetime import datetime


class Template:
    def __init__(self, name, path):
        self.name = name
        self.path = path

    def write(self):
        path = self.path
        name = self.name
        with open(os.path.join(path, "pyproject.toml"), "w") as f:
            f.write(self._content())
        with open(os.path.join(path, "License"), "w") as f:
            f.write(self._mit_license())
        with open(os.path.join(path, "README.md"), "w") as f:
            f.write(self._readme())
        with open(os.path.join(path, "src", name, "cli.py"), "w") as f:
            f.write(self._cli())

    def build(self):
        path = self.path
        name = self.name
        is_exists = os.path.exists(path=path)
        if is_exists:
            raise NameError(colored(f"Directory {path} has already exists", "red"))
        os.makedirs(path)
        os.makedirs(os.path.join(path, "src", name))
        self.write()

    def _content(self):
        name = self.name
        return f"""[build-system]
requires = ["setuptools"]
build-backend = "setuptools.build_meta"


[project]
name = "{name}"
authors = [{{ name = "YOUR_NAME", email = "YOUR_EMAIL@ADDRESS.COM" }}]
description = "YOUR DESCRIPTION HERE"
readme = "README.md"
requires-python = ">=3.6"
license = {{ text = "MIT" }}
version = "0.0.1"
keywords = ["SOME_KEYWORDS"]
classifiers = ["Programming Language :: Python :: 3"]
dependencies = ['importlib-metadata; python_version>"3.6"']

[tool.setuptools.packages.find]
where = ["src"]

# Uncomment to export script file in bin
# [tool.setuptools]
# script-files = ['bin/{name}']

[project.urls]
Homepage = "https://github.com/YOUR_NAME/{name}"
Repository = "https://github.com/YOUR_NAME/{name}"
Documentation = "https://github.com/YOUR_NAME/{name}"

# Uncomment to use scripts
# [project.scripts]
# {name} = "{name}.cli:cli"
"""

    @staticmethod
    def _mit_license():
        return f"""MIT License

Copyright (c) [{datetime.now().year}] [YOUR_NAME]

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

    def _readme(self):
        return f"""# {self.name}

Hello, {self.name}.
"""

    def _cli(self):
        return f"""def cli():
    print("Hello, {self.name}")

if __name__ == "__main__":
    cli()
"""



import os
import re

def wrap():
    if os.name != 'nt':
        home_dir = os.path.expanduser("~")
        profile = os.path.join(home_dir, '.profile')
        if not os.path.exists(profile):
            with open(profile, 'w') as f:
                f.write("# Empty profile file.")
        
        with open(profile, 'r') as f:
            text = f.read()
            regex = r"# pvw wrapper start\n(.+?)\n# pvw wrapper end"
            with open('magic.sh') as m:
                content = m.read()
                if re.search(regex, text, re.DOTALL):
                    new_text = re.sub(regex, content, text, flags=re.DOTALL)
                else:
                    new_text = text + f"\n\n{content}"
        
        with open(profile, 'w') as f:
            f.write(new_text)

        os.system(f". {profile}")



if __name__ == '__main__':
    wrap()


import subprocess
import sys
import os
import shutil


class ShellExecutor:
    def __init__(self) -> None:
        self.on_win = sys.platform.startswith("win")
        self.py = self.check_python_installation()
        self.pipe_name = os.path.join(os.path.expanduser("~"), ".pvw", "_envs._cfg")

    @staticmethod
    def run(cmds: [str], shell=False):
        subprocess.run(
            cmds,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
            shell=shell,
        )

    # Config
    def _is_vim_installed(self):
        try:
            self.run(["vim", "--version"])
            return True
        except Exception:
            return False

    def edit_file(self, path):
        if self._is_vim_installed():
            subprocess.run(["vim", path])
        else:
            print("Hint: vim is not installed.")
            print(
                """You can install vim, or use `pvw config get` or `pvw config set` directly.
e.g. `pvw config set venv_path=/PATH/TO/PLACE/VENVS`"""
            )

    def check_python_installation(self):
        try:
            self.run(["python", "--version"])
            return "python"
        except Exception:
            try:
                self.run(["python3", "--version"])
                return "python3"
            except Exception:
                raise NameError("python or python3 is not detected.")

    def save_to_pipe(self, msg):
        pass

    # Environments
    def create_env(self, path):
        self.run(f"{self.py} -m venv {path}", shell=True)

    def activate_env(self, path):
        if self.on_win:
            script_path = os.path.join(path, "Scripts", "Activate.ps1")
        else:
            script_path = os.path.join(path, "bin", "activate")
        # Use file as a pipe to communicate with parent process.
        with open(self.pipe_name, "w") as f:
            f.write(script_path)

    def remove_env(self, path):
        shutil.rmtree(path)

    def replace_path(self, file_rel_path, source_path, target_path):
        content = ""
        source_prompt = f"({os.path.basename(source_path)})"
        target_prompt = f"({os.path.basename(target_path)})"
        with open(os.path.join(source_path, file_rel_path), "r") as f:
            content = (
                f.read()
                .replace(source_path, target_path)
                .replace(source_prompt, target_prompt)
            )
            with open(os.path.join(target_path, file_rel_path), "w") as f:
                f.write(content)

    def copy_env(self, source_path, target_path):
        self.create_env(target_path)
        if self.on_win:
            script_path = "Scripts"
            shutil.copytree(
                source_path,
                target_path,
                ignore=shutil.ignore_patterns("python*.exe", "pip*.exe"),
                dirs_exist_ok=True,
            )
        else:
            script_path = "bin"
            shutil.copytree(
                source_path,
                target_path,
                ignore=shutil.ignore_patterns(script_path),
                dirs_exist_ok=True,
            )
            shutil.copytree(
                os.path.join(source_path, script_path),
                os.path.join(target_path, script_path),
                ignore=shutil.ignore_patterns("python*", "pip*"),
                dirs_exist_ok=True,
            )

        self.replace_path("pyvenv.cfg", source_path, target_path)
        self.replace_path(
            os.path.join(script_path, "activate"), source_path, target_path
        )
        if self.on_win:
            self.replace_path(
                os.path.join(script_path, "activate.bat"), source_path, target_path
            )



from setuptools import setup
# from setuptools.command.install import install
# import os


# class PvwWrapperInstall(install):
#     def run(self):
#         if os.name != "nt":
#             self.wrapper_install()
#         install.run(self)

#     def wrapper_install(self):
#         pass


# setup(cmdclass={"install": PvwWrapperInstall})
setup()



import os
import json
from pvw.shell_executor import ShellExecutor


class Config:
    def __init__(self):
        user_path = os.path.expanduser("~")
        config_dir = os.path.join(user_path, ".pvw", "config")
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)
        self.config_path = os.path.join(config_dir, "config.json")
        if not os.path.exists(self.config_path):
            path = self.ask_for_init()
            self.config = {"venv_path": path}
            try:
                self.save()
                print(f"Create config file in {self.config_path}")
            except Exception as e:
                raise NameError(f"invalid venv path {path}.")
        else:
            with open(self.config_path, "r") as f:
                self.config = json.load(f)
        self.check_venv_path()
        self.shell = ShellExecutor()

    def edit(self):
        self.shell.edit_file(self.config_path)

    def get(self, key):
        try:
            return self.config[key]
        except KeyError:
            print(f"No config option named {key}")

    def set(self, key, value):
        if key in self.config:
            self.config[key] = value
            self.save()

    def add_venv(self, name, path):
        self.config["venvs"].append(
            {
                "name": name,
                "path": path,
            }
        )
        self.save()

    def remove_venv(self, names):
        venvs = [info for info in self.config["venvs"] if info["name"] not in names]
        self.config["venvs"] = venvs
        self.save()        

    def check_venv_path(self):
        try:
            if "venvs" not in self.config:
                self.config["venvs"] = []
                path = self.config["venv_path"]
                for env in os.listdir(path):
                    if env.startswith("."):
                        continue
                    self.config["venvs"].append(
                        {
                            "name": env,
                            "path": os.path.join(path, env),
                        }
                    )
                self.save()
            else:
                venvs = self.config["venvs"]
                updated_venvs = [env for env in venvs if os.path.exists(env["path"])]
                if len(venvs) != len(updated_venvs):
                    self.config["venvs"] = updated_venvs
                    self.save()

        except Exception as e:
            print(e)

    def save(self):
        with open(self.config_path, "w") as f:
            json.dump(self.config, f)

    @property
    def venv_path(self):
        try:
            with open(self.config_path) as f:
                config = json.load(f)
                path = config["venv_path"]
                if path.startswith("~/"):
                    path = os.path.join(os.path.expanduser("~"), path[2:])
                return path
        except Exception as e:
            print(e)

    def build_venv_path(self):
        if not os.path.exists(self.venv_path):
            os.makedirs(self.venv_path)
            print(f"create directory for venvs: {self.venv_path}")

    def ask_for_init(self):
        path = input("Set the directory for venv (~/venvs):")
        if not path:
            path = "~/venvs"
        if path.startswith("~/"):
            path = os.path.join(os.path.expanduser("~"), path[2:])
        return path


try:
    config = Config()
except Exception:
    raise InterruptedError("Config initialized interrupted.")
